@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
@@ -29,8 +29,6 @@
 #include "pango-fontmap.h"
 #include "pango-impl-utils.h"
 
-static const char bad_font_warning[] = "%s called with null font argument, expect ugly output";
-
 struct _PangoFontDescription
 {
   char *family_name;
@@ -53,7 +51,7 @@ pango_font_description_get_type (void)
 {
   static GType our_type = 0;
 
-  if (our_type == 0)
+  if (G_UNLIKELY (our_type == 0))
     our_type = g_boxed_type_register_static (I_("PangoFontDescription"),
 					     (GBoxedCopyFunc)pango_font_description_copy,
 					     (GBoxedFreeFunc)pango_font_description_free);
@@ -657,19 +655,21 @@ pango_font_description_better_match (const PangoFontDescription *desc,
 
 /**
  * pango_font_description_copy:
- * @desc: a #PangoFontDescription
+ * @desc: a #PangoFontDescription, may be %NULL
  *
  * Make a copy of a #PangoFontDescription.
  *
  * Return value: the newly allocated #PangoFontDescription, which should
- *               be freed with pango_font_description_free().
+ *               be freed with pango_font_description_free(), or %NULL
+ *               if @desc was %NULL.
  **/
 PangoFontDescription *
 pango_font_description_copy  (const PangoFontDescription  *desc)
 {
   PangoFontDescription *result;
 
-  g_return_val_if_fail (desc != NULL, NULL);
+  if (desc == NULL)
+    return NULL;
 
   result = g_slice_new (PangoFontDescription);
 
@@ -686,7 +686,7 @@ pango_font_description_copy  (const PangoFontDescription  *desc)
 
 /**
  * pango_font_description_copy_static:
- * @desc: a #PangoFontDescription
+ * @desc: a #PangoFontDescription, may be %NULL
  *
  * Like pango_font_description_copy(), but only a shallow copy is made
  * of the family name and other allocated fields. The result can only
@@ -694,14 +694,16 @@ pango_font_description_copy  (const PangoFontDescription  *desc)
  * when the copy is only needed temporarily.
  *
  * Return value: the newly allocated #PangoFontDescription, which should
- *               be freed with pango_font_description_free().
+ *               be freed with pango_font_description_free(), or %NULL
+ *               if @desc was %NULL.
  **/
 PangoFontDescription *
 pango_font_description_copy_static (const PangoFontDescription *desc)
 {
   PangoFontDescription *result;
 
-  g_return_val_if_fail (desc != NULL, NULL);
+  if (desc == NULL)
+    return NULL;
 
   result = g_slice_new (PangoFontDescription);
 
@@ -794,25 +796,25 @@ pango_font_description_hash (const PangoFontDescription *desc)
 
 /**
  * pango_font_description_free:
- * @desc: a #PangoFontDescription, or %NULL
+ * @desc: a #PangoFontDescription, may be %NULL
  *
  * Frees a font description.
  **/
 void
 pango_font_description_free  (PangoFontDescription  *desc)
 {
-  if (desc)
-    {
-      if (desc->family_name && !desc->static_family)
-	g_free (desc->family_name);
+  if (desc == NULL)
+    return;
 
-      g_slice_free (PangoFontDescription, desc);
-    }
+  if (desc->family_name && !desc->static_family)
+    g_free (desc->family_name);
+
+  g_slice_free (PangoFontDescription, desc);
 }
 
 /**
  * pango_font_descriptions_free:
- * @descs: a pointer to an array of #PangoFontDescription, or %NULL
+ * @descs: a pointer to an array of #PangoFontDescription, may be %NULL
  * @n_descs: number of font descriptions in @descs
  *
  * Frees a list of font descriptions from pango_font_map_list_fonts()
@@ -823,12 +825,12 @@ pango_font_descriptions_free (PangoFontDescription **descs,
 {
   int i;
 
-  if (descs)
-    {
-      for (i = 0; i<n_descs; i++)
-	pango_font_description_free (descs[i]);
-      g_free (descs);
-    }
+  if (descs == NULL)
+    return;
+
+  for (i = 0; i<n_descs; i++)
+    pango_font_description_free (descs[i]);
+  g_free (descs);
 }
 
 typedef struct
@@ -882,6 +884,34 @@ static const FieldMap gravity_map[] = {
   { PANGO_GRAVITY_WEST,  "West" }
 };
 
+static gboolean
+field_matches (const gchar *s1,
+	       const gchar *s2,
+	       gsize n)
+{
+  gint c1, c2;
+
+  g_return_val_if_fail (s1 != NULL, 0);
+  g_return_val_if_fail (s2 != NULL, 0);
+
+  while (n && *s1 && *s2)
+    {
+      c1 = (gint)(guchar) TOLOWER (*s1);
+      c2 = (gint)(guchar) TOLOWER (*s2);
+      if (c1 != c2) {
+        if (c1 == '-') {
+	  s1++;
+	  continue;
+	}
+	return FALSE;
+      }
+      s1++; s2++;
+      n--;
+    }
+
+  return n == 0 && *s1 == '\0';
+}
+
 static gboolean
 find_field (const FieldMap *map, int n_elements, const char *str, int len, int *val)
 {
@@ -889,8 +919,7 @@ find_field (const FieldMap *map, int n_elements, const char *str, int len, int *
 
   for (i=0; i<n_elements; i++)
     {
-      if (map[i].str[0] && g_ascii_strncasecmp (map[i].str, str, len) == 0 &&
-	  map[i].str[len] == '\0')
+      if (map[i].str[0] && field_matches (map[i].str, str, len))
 	{
 	  if (val)
 	    *val = map[i].value;
@@ -1195,7 +1224,7 @@ pango_font_description_to_filename (const PangoFontDescription  *desc)
   return result;
 }
 
-G_DEFINE_TYPE (PangoFont, pango_font, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE (PangoFont, pango_font, G_TYPE_OBJECT)
 
 static void
 pango_font_class_init (PangoFontClass *class)
@@ -1310,6 +1339,9 @@ pango_font_find_shaper (PangoFont     *font,
  * PANGO_DESCENT(), PANGO_LBEARING(), and PANGO_RBEARING() can be used to convert
  * from the extents rectangle to more traditional font metrics. The units
  * of the rectangles are in 1/PANGO_SCALE of a device unit.
+ *
+ * If @font is %NULL, this function gracefully sets some sane values in the
+ * output variables and returns.
  **/
 void
 pango_font_get_glyph_extents  (PangoFont      *font,
@@ -1319,12 +1351,6 @@ pango_font_get_glyph_extents  (PangoFont      *font,
 {
   if (G_UNLIKELY (!font))
     {
-
-      if (!_pango_warning_history.get_glyph_extents)
-	{
-	  _pango_warning_history.get_glyph_extents = TRUE;
-	  g_warning (bad_font_warning, "pango_font_get_glyph_extents");
-	}
       if (ink_rect)
 	{
 	  ink_rect->x = PANGO_SCALE;
@@ -1357,6 +1383,9 @@ pango_font_get_glyph_extents  (PangoFont      *font,
  * be provided to indicate that the metrics should be retrieved that
  * correspond to the script(s) used by that language.
  *
+ * If @font is %NULL, this function gracefully sets some sane values in the
+ * output variables and returns.
+ *
  * Return value: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
  *   when finished using the object.
  **/
@@ -1366,14 +1395,7 @@ pango_font_get_metrics (PangoFont        *font,
 {
   if (G_UNLIKELY (!font))
     {
-      PangoFontMetrics *metrics;
-
-      if (!_pango_warning_history.get_metrics)
-	{
-	  _pango_warning_history.get_metrics = TRUE;
-	  g_warning (bad_font_warning, "pango_font_get_metrics");
-	}
-      metrics = pango_font_metrics_new ();
+      PangoFontMetrics *metrics = pango_font_metrics_new ();
 
       metrics->ascent = PANGO_SCALE * PANGO_UNKNOWN_GLYPH_HEIGHT;
       metrics->descent = 0;
@@ -1392,11 +1414,19 @@ pango_font_get_metrics (PangoFont        *font,
 
 /**
  * pango_font_get_font_map:
- * @font: a #PangoFont
+ * @font: a #PangoFont, or %NULL
  *
  * Gets the font map for which the font was created.
  *
- * Return value: the #PangoFontMap for the font
+ * Note that the font maintains a <firstterm>weak</firstterm> reference
+ * to the font map, so if all references to font map are dropped, the font
+ * map will be finalized even if there are fonts created with the font
+ * map that are still alive.  In that case this function will return %NULL.
+ * It is the responsibility of the user to ensure that the font map is kept
+ * alive.  In most uses this is not an issue as a #PangoContext holds
+ * a reference to the font map.
+ *
+ * Return value: the #PangoFontMap for the font, or %NULL if @font is %NULL.
  *
  * Since: 1.10
  **/
@@ -1404,15 +1434,7 @@ PangoFontMap *
 pango_font_get_font_map (PangoFont *font)
 {
   if (G_UNLIKELY (!font))
-    {
-
-      if (!_pango_warning_history.get_font_map)
-	{
-	  _pango_warning_history.get_font_map = TRUE;
-	  g_warning (bad_font_warning, "pango_font_get_font_map");
-	}
-      return NULL;
-    }
+    return NULL;
 
   if (PANGO_FONT_GET_CLASS (font)->get_font_map)
     return PANGO_FONT_GET_CLASS (font)->get_font_map (font);
@@ -1425,7 +1447,7 @@ pango_font_metrics_get_type (void)
 {
   static GType our_type = 0;
 
-  if (our_type == 0)
+  if (G_UNLIKELY (our_type == 0))
     our_type = g_boxed_type_register_static (I_("PangoFontMetrics"),
 					     (GBoxedCopyFunc)pango_font_metrics_ref,
 					     (GBoxedFreeFunc)pango_font_metrics_unref);
@@ -1455,7 +1477,7 @@ pango_font_metrics_new (void)
 
 /**
  * pango_font_metrics_ref:
- * @metrics: a #PangoFontMetrics structure
+ * @metrics: a #PangoFontMetrics structure, may be %NULL
  *
  * Increase the reference count of a font metrics structure by one.
  *
@@ -1464,17 +1486,17 @@ pango_font_metrics_new (void)
 PangoFontMetrics *
 pango_font_metrics_ref (PangoFontMetrics *metrics)
 {
-  if (!metrics)
+  if (metrics == NULL)
     return NULL;
 
-  metrics->ref_count++;
+  g_atomic_int_inc (&metrics->ref_count);
 
   return metrics;
 }
 
 /**
  * pango_font_metrics_unref:
- * @metrics: a #PangoFontMetrics structure
+ * @metrics: a #PangoFontMetrics structure, may be %NULL
  *
  * Decrease the reference count of a font metrics structure by one. If
  * the result is zero, frees the structure and any associated
@@ -1483,13 +1505,12 @@ pango_font_metrics_ref (PangoFontMetrics *metrics)
 void
 pango_font_metrics_unref (PangoFontMetrics *metrics)
 {
-  if (!metrics)
+  if (metrics == NULL)
     return;
+
   g_return_if_fail (metrics->ref_count > 0 );
 
-  metrics->ref_count--;
-
-  if (metrics->ref_count == 0)
+  if (g_atomic_int_dec_and_test (&metrics->ref_count))
     g_slice_free (PangoFontMetrics, metrics);
 }
 
@@ -1655,7 +1676,7 @@ pango_font_metrics_get_strikethrough_thickness (PangoFontMetrics *metrics)
  * PangoFontFamily
  */
 
-G_DEFINE_TYPE (PangoFontFamily, pango_font_family, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE (PangoFontFamily, pango_font_family, G_TYPE_OBJECT)
 
 static void
 pango_font_family_class_init (PangoFontFamilyClass *class)
@@ -1744,7 +1765,7 @@ pango_font_family_is_monospace (PangoFontFamily  *family)
  * PangoFontFace
  */
 
-G_DEFINE_TYPE (PangoFontFace, pango_font_face, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE (PangoFontFace, pango_font_face, G_TYPE_OBJECT)
 
 static void
 pango_font_face_class_init (PangoFontFaceClass *class)
