@@ -1,15 +1,28 @@
-/*******************************************************************
- *
- *  Copyright 1996-2000 by
- *  David Turner, Robert Wilhelm, and Werner Lemberg.
- *
- *  Copyright 2006  Behdad Esfahbod
+/*
+ * Copyright (C) 1998-2004  David Turner and Werner Lemberg
+ * Copyright (C) 2006  Behdad Esfahbod
  *
  *  This is part of HarfBuzz, an OpenType Layout engine library.
  *
- *  See the file name COPYING for licensing information.
+ * Permission is hereby granted, without written agreement and without
+ * license or royalty fees, to use, copy, modify, and distribute this
+ * software and its documentation for any purpose, provided that the
+ * above copyright notice and the following two paragraphs appear in
+ * all copies of this software.
  *
- ******************************************************************/
+ * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
+ * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
+ * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
+ * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
+ * DAMAGE.
+ *
+ * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
+ * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+ * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
+ * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
+ * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
+ */
+
 #ifndef HARFBUZZ_GPOS_H
 #define HARFBUZZ_GPOS_H
 
@@ -41,11 +54,9 @@ HB_BEGIN_HEADER
 
    If for some reason no outline data is available (e.g. for an
    embedded bitmap glyph), _glyph->outline.n_points should be set to
-   zero.  _glyph can be computed with
+   zero.                                                                */
 
-      _glyph = HANDLE_Glyph( glyph )                                    */
-
-typedef HB_Error  (*HB_GlyphFunction)(FT_Face      face,
+typedef HB_Error  (*HB_GlyphFunction)(HB_Font      font,
 				       HB_UInt      glyphIndex,
 				       HB_Int       loadFlags );
 
@@ -61,15 +72,15 @@ typedef HB_Error  (*HB_GlyphFunction)(FT_Face      face,
    `metric_value' must be returned as a scaled value (but shouldn't
    be rounded).                                                       */
 
-typedef HB_Error  (*HB_MMFunction)(FT_Face      face,
+typedef HB_Error  (*HB_MMFunction)(HB_Font      font,
 				    HB_UShort    metric_id,
-				    FT_Pos*      metric_value,
+				    HB_Fixed*      metric_value,
 				    void*        data );
 
 
 struct  HB_GPOSHeader_
 {
-  FT_Fixed           Version;
+  HB_16Dot16           Version;
 
   HB_ScriptList     ScriptList;
   HB_FeatureList    FeatureList;
@@ -94,7 +105,7 @@ typedef struct HB_GPOSHeader_  HB_GPOSHeader;
 typedef HB_GPOSHeader* HB_GPOS;
 
 
-HB_Error  HB_Load_GPOS_Table( FT_Face          face,
+HB_Error  HB_Load_GPOS_Table( HB_Font          font,
 			      HB_GPOSHeader** gpos,
 			      HB_GDEFHeader*  gdef );
 
@@ -151,7 +162,7 @@ HB_Error  HB_GPOS_Register_MM_Function( HB_GPOSHeader*  gpos,
    tables are ignored -- you will get device independent values.         */
 
 
-HB_Error  HB_GPOS_Apply_String( FT_Face           face,
+HB_Error  HB_GPOS_Apply_String( HB_Font           font,
 				HB_GPOSHeader*   gpos,
 				HB_UShort         load_flags,
 				HB_Buffer        buffer,
