@@ -19,11 +19,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <math.h>
 
 #include "pangocairo-private.h"
+#include "pango-glyph-item.h"
 
 typedef struct _PangoCairoRendererClass PangoCairoRendererClass;
 
@@ -37,6 +38,7 @@ struct _PangoCairoRenderer
 
   cairo_t *cr;
   gboolean do_path;
+  gboolean has_show_text_glyphs;
   double x_offset, y_offset;
 
   /* house-keeping options */
@@ -270,22 +272,30 @@ done:
   cairo_restore (crenderer->cr);
 }
 
+#ifndef STACK_BUFFER_SIZE
+#define STACK_BUFFER_SIZE (512 * sizeof (int))
+#endif
+
+#define STACK_ARRAY_LENGTH(T) (STACK_BUFFER_SIZE / sizeof(T))
+
 static void
-pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
-				  PangoFont         *font,
-				  PangoGlyphString  *glyphs,
-				  int                x,
-				  int                y)
+pango_cairo_renderer_show_text_glyphs (PangoRenderer        *renderer,
+				       const char           *text,
+				       int                   text_len,
+				       PangoGlyphString     *glyphs,
+				       cairo_text_cluster_t *clusters,
+				       int                   num_clusters,
+				       gboolean              backward,
+				       PangoFont            *font,
+				       int                   x,
+				       int                   y)
 {
   PangoCairoRenderer *crenderer = (PangoCairoRenderer *) (renderer);
 
-  /* cairo_glyph_t is 24 bytes */
-#define MAX_STACK 40
-
   int i, count;
   int x_position = 0;
   cairo_glyph_t *cairo_glyphs;
-  cairo_glyph_t stack_glyphs[MAX_STACK];
+  cairo_glyph_t stack_glyphs[STACK_ARRAY_LENGTH (cairo_glyph_t)];
   double base_x = crenderer->x_offset + (double)x / PANGO_SCALE;
   double base_y = crenderer->y_offset + (double)y / PANGO_SCALE;
 
@@ -314,7 +324,7 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
       goto done;
     }
 
-  if (glyphs->num_glyphs > MAX_STACK)
+  if (glyphs->num_glyphs > (int) G_N_ELEMENTS (stack_glyphs))
     cairo_glyphs = g_new (cairo_glyph_t, glyphs->num_glyphs);
   else
     cairo_glyphs = stack_glyphs;
@@ -344,18 +354,118 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
       x_position += gi->geometry.width;
     }
 
-  if (crenderer->do_path)
+  if (G_UNLIKELY (crenderer->do_path))
     cairo_glyph_path (crenderer->cr, cairo_glyphs, count);
   else
-    cairo_show_glyphs (crenderer->cr, cairo_glyphs, count);
+    if (G_UNLIKELY (clusters))
+      cairo_show_text_glyphs (crenderer->cr,
+			      text, text_len,
+			      cairo_glyphs, count,
+			      clusters, num_clusters,
+			      backward ? CAIRO_TEXT_CLUSTER_FLAG_BACKWARD : 0);
+    else
+      cairo_show_glyphs (crenderer->cr, cairo_glyphs, count);
 
-  if (glyphs->num_glyphs > MAX_STACK)
+  if (cairo_glyphs != stack_glyphs)
     g_free (cairo_glyphs);
 
 done:
   cairo_restore (crenderer->cr);
+}
 
-#undef MAX_STACK
+static void
+pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
+				  PangoFont         *font,
+				  PangoGlyphString  *glyphs,
+				  int                x,
+				  int                y)
+{
+  pango_cairo_renderer_show_text_glyphs (renderer,
+					 NULL, 0,
+					 glyphs,
+					 NULL, 0,
+					 FALSE,
+					 font,
+					 x, y);
+}
+
+static void
+pango_cairo_renderer_draw_glyph_item (PangoRenderer     *renderer,
+				      const char        *text,
+				      PangoGlyphItem    *glyph_item,
+				      int                x,
+				      int                y)
+{
+  PangoCairoRenderer *crenderer = (PangoCairoRenderer *) (renderer);
+  PangoFont          *font      = glyph_item->item->analysis.font;
+  PangoGlyphString   *glyphs    = glyph_item->glyphs;
+  PangoItem          *item      = glyph_item->item;
+  gboolean            backward  = (item->analysis.level & 1) != 0;
+
+  PangoGlyphItemIter   iter;
+  cairo_text_cluster_t *cairo_clusters;
+  cairo_text_cluster_t stack_clusters[STACK_ARRAY_LENGTH (cairo_text_cluster_t)];
+  int num_clusters;
+
+  if (!crenderer->has_show_text_glyphs || crenderer->do_path)
+    {
+      pango_cairo_renderer_show_text_glyphs (renderer,
+					     NULL, 0,
+					     glyphs,
+					     NULL, 0,
+					     FALSE,
+					     font,
+					     x, y);
+      return;
+    }
+
+  if (glyphs->num_glyphs > (int) G_N_ELEMENTS (stack_clusters))
+    cairo_clusters = g_new (cairo_text_cluster_t, glyphs->num_glyphs);
+  else
+    cairo_clusters = stack_clusters;
+
+  num_clusters = 0;
+  if (pango_glyph_item_iter_init_start (&iter, glyph_item, text))
+    {
+      do {
+        int num_bytes, num_glyphs, i;
+
+        num_bytes  = iter.end_index - iter.start_index;
+        num_glyphs = backward ? iter.start_glyph - iter.end_glyph : iter.end_glyph - iter.start_glyph;
+
+	if (num_bytes < 1)
+	  g_warning ("pango_cairo_renderer_draw_glyph_item: bad cluster has num_bytess %d", num_bytes);
+	if (num_glyphs < 1)
+	  g_warning ("pango_cairo_renderer_draw_glyph_item: bad cluster has num_glyphs %d", num_glyphs);
+
+	/* Discount empty and unknown glyphs */
+	for (i = MIN (iter.start_glyph, iter.end_glyph+1);
+	     i < MAX (iter.start_glyph+1, iter.end_glyph);
+	     i++)
+	  {
+	    PangoGlyphInfo *gi = &glyphs->glyphs[i];
+
+	    if (gi->glyph == PANGO_GLYPH_EMPTY ||
+		gi->glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+	      num_glyphs--;
+	  }
+
+        cairo_clusters[num_clusters].num_bytes  = num_bytes;
+        cairo_clusters[num_clusters].num_glyphs = num_glyphs;
+        num_clusters++;
+      } while (pango_glyph_item_iter_next_cluster (&iter));
+    }
+
+  pango_cairo_renderer_show_text_glyphs (renderer,
+					 text + item->offset, item->length,
+					 glyphs,
+					 cairo_clusters, num_clusters,
+					 backward,
+					 font,
+					 x, y);
+
+  if (cairo_clusters != stack_clusters)
+    g_free (cairo_clusters);
 }
 
 static void
@@ -544,6 +654,7 @@ pango_cairo_renderer_class_init (PangoCairoRendererClass *klass)
   PangoRendererClass *renderer_class = PANGO_RENDERER_CLASS (klass);
 
   renderer_class->draw_glyphs = pango_cairo_renderer_draw_glyphs;
+  renderer_class->draw_glyph_item = pango_cairo_renderer_draw_glyph_item;
   renderer_class->draw_rectangle = pango_cairo_renderer_draw_rectangle;
   renderer_class->draw_error_underline = pango_cairo_renderer_draw_error_underline;
   renderer_class->draw_shape = pango_cairo_renderer_draw_shape;
@@ -582,6 +693,7 @@ release_renderer (PangoCairoRenderer *renderer)
     {
       renderer->cr = NULL;
       renderer->do_path = FALSE;
+      renderer->has_show_text_glyphs = FALSE;
       renderer->x_offset = 0.;
       renderer->y_offset = 0.;
 
@@ -596,6 +708,9 @@ save_current_point (PangoCairoRenderer *renderer)
 {
   renderer->cr_had_current_point = cairo_has_current_point (renderer->cr);
   cairo_get_current_point (renderer->cr, &renderer->x_offset, &renderer->y_offset);
+
+  /* abuse save_current_point() to cache cairo_has_show_text_glyphs() result */
+  renderer->has_show_text_glyphs = cairo_surface_has_show_text_glyphs (cairo_get_target (renderer->cr));
 }
 
 static void
@@ -651,6 +766,45 @@ _pango_cairo_do_glyph_string (cairo_t          *cr,
   release_renderer (crenderer);
 }
 
+static void
+_pango_cairo_do_glyph_item (cairo_t          *cr,
+			    const char       *text,
+			    PangoGlyphItem   *glyph_item,
+			    gboolean          do_path)
+{
+  PangoCairoRenderer *crenderer = acquire_renderer ();
+  PangoRenderer *renderer = (PangoRenderer *) crenderer;
+
+  crenderer->cr = cr;
+  crenderer->do_path = do_path;
+  save_current_point (crenderer);
+
+  if (!do_path)
+    {
+      /* unset all part colors, since when drawing just a glyph string,
+       * prepare_run() isn't called.
+       */
+
+      pango_renderer_activate (renderer);
+
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_FOREGROUND, NULL);
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, NULL);
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, NULL);
+      pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, NULL);
+    }
+
+  pango_renderer_draw_glyph_item (renderer, text, glyph_item, 0, 0);
+
+  if (!do_path)
+    {
+      pango_renderer_deactivate (renderer);
+    }
+
+  restore_current_point (crenderer);
+
+  release_renderer (crenderer);
+}
+
 static void
 _pango_cairo_do_layout_line (cairo_t          *cr,
 			     PangoLayoutLine  *line,
@@ -739,6 +893,38 @@ pango_cairo_show_glyph_string (cairo_t          *cr,
   _pango_cairo_do_glyph_string (cr, font, glyphs, FALSE);
 }
 
+
+/**
+ * pango_cairo_show_glyph_item:
+ * @cr: a Cairo context
+ * @text: the UTF-8 text that @glyph_item refers to
+ * @glyph_item: a #PangoGlyphItem
+ *
+ * Draws the glyphs in @glyph_item in the specified cairo context,
+ * embedding the text associated with the glyphs in the output if the
+ * output format supports it (PDF for example), otherwise it acts
+ * similar to pango_cairo_show_glyph_string().
+ *
+ * The origin of the glyphs (the left edge of the baseline) will
+ * be drawn at the current point of the cairo context.
+ *
+ * Note that @text is the start of the text for layout, which is then
+ * indexed by <literal>@glyph_item->item->offset</literal>.
+ *
+ * Since: 1.22
+ **/
+void
+pango_cairo_show_glyph_item (cairo_t          *cr,
+			     const char       *text,
+			     PangoGlyphItem   *glyph_item)
+{
+  g_return_if_fail (cr != NULL);
+  g_return_if_fail (text != NULL);
+  g_return_if_fail (glyph_item != NULL);
+
+  _pango_cairo_do_glyph_item (cr, text, glyph_item, FALSE);
+}
+
 /**
  * pango_cairo_show_layout_line:
  * @cr: a Cairo context
